Record domain : Type := {
  domain_set : Set;
  domain_le : domain_set -> domain_set -> Prop;
  domain_le_refl : forall x : domain_set, (domain_le x x);
  domain_le_trans : forall x y z : domain_set,
    (domain_le x y) -> (domain_le y z) -> (domain_le x z)
}.

Section Product_domain.
Variable T1 T2 : domain.

Definition product_set : Set := (prod (domain_set T1) (domain_set T2)).
Definition product_le (x : product_set) (y : product_set) : Prop :=
  (domain_le T1 (fst x) (fst y)) /\ (domain_le T2 (snd x) (snd y)).

Lemma product_le_refl : forall x : product_set, (product_le x x).
Proof.
  destruct x.
  unfold product_le in |- *.
  simpl in |- *.
  split.
   apply (domain_le_refl T1).
   apply (domain_le_refl T2).
Qed.

Lemma product_le_trans : forall x y z : product_set,
  (product_le x y) -> (product_le y z) -> (product_le x z).
Proof.
  destruct x; destruct y; destruct z.
  unfold product_le in |- *; simpl in |- *.
  intuition.
   apply (domain_le_trans T1) with (y := d1); assumption.
   apply (domain_le_trans T2) with (y := d2); assumption.
Qed.

Definition product_domain : domain := Build_domain product_set product_le product_le_refl product_le_trans.

End Product_domain.

Record edomain : Type := {
  mdomain : domain;
  edomain_le :
    forall x y : domain_set mdomain,
      { (domain_le mdomain) x y } + {~ ((domain_le mdomain) x y) } }.

Section Product_edomain.
Variable T1 T2 : edomain.

Definition product_mdomain : domain :=
  (product_domain (mdomain T1) (mdomain T2)).

Definition eproduct_le :
  forall x y : (domain_set product_mdomain),
    { (domain_le product_mdomain) x y } +
    {~ ((domain_le product_mdomain) x y) }.
Proof.
  intros.
  destruct (edomain_le T1 (fst x) (fst y)).
   destruct (edomain_le T2 (snd x) (snd y)).
    left.
      simpl in |- *.
      unfold product_le in |- *.
      tauto.
    right.
      simpl in |- *.
      unfold product_le in |- *.
      tauto.
   right.
     simpl in |- *.
     unfold product_le in |- *.
     tauto.
Qed.

Definition eproduct_domain := Build_edomain product_mdomain eproduct_le.
End Product_edomain.

Extraction Inline eproduct_le.
